/* EPS Conduits (C) 2014-2017 Stephane Charette <stephanecharette@gmail.com>
 * $Id: ServerHull.hpp 2167 2017-03-01 12:10:04Z stephane $
 * See license.txt.
 */

#pragma once

#include "Hull.hpp"
#include "Tracker.hpp"
#include "comm.hpp"
#include <boost/asio/basic_socket_acceptor.hpp>


namespace EPS
{
	/// forward declaration
	//class TConnPtr;

	typedef std::map<EPS::UUID, CommStats1> MUuidStats;
	typedef std::map<EPS::UUID, EPS::UUIDSet> MUuidMesh;

	/// Server hull. @see EPS::Hull @see EPS::ClientHull
	class ServerHull final : public Hull
	{
		public:

			/// Constructor.
			ServerHull(int argc, char *argv[]);

			/// Destructor.
			virtual ~ServerHull(void);

			/// Run %EPS server.
			virtual void run_second_stage(void);

			/// Setup the IPv4 address pool from which we assign an address to each client.
			void setup_address_pool(void);

			/// Begin the async timer that eventually calls @ref generate_network_diagram().  This is triggered every 60 seconds.
			void start_graphviz_network_diagram_timer(void);

			/// Generate a Graphviz-format network diagram.
			void generate_network_diagram(void);

			/// Callback when a connection has failed and needs to be cleaned up.
			virtual void connection_failed(const EPS::TConnPtr &connection);

			/// Called when a communication command is received.
			virtual void processComm(const EPS::TConnPtr &connection, EPS::NetBuffer *nb, const EPS::CommHeader1 *header, const EPS::ECommType type);

#ifndef EPS_SKIP_UPNP

			/// Setup UPNP (port forwarding).
			void setup_UPnP(void);

			/// Reset UPNP.
			void reset_UPnP(void);

			/// Parse and identify relevant UPnP mappings.
			void read_UPnP_mappings(void);

			/// Renew the UPnP mappings.
			void renew_UPnP_mappings(void);

			/// Delete UPNP port mapping.
			bool delete_UPnP_mapping(const std::string &protocol);

			/// Create UPNP port mapping.
			bool create_UPnP_mapping(const std::string &protocol);

#endif

			// members

			boost::asio::system_timer	graphviz_network_diagram_timer;	///< timer triggered every 60 seconds to generate a graphviz network diagram

			MUuidStats	uuid_stats;	///< individual statistics, stored in a map by UUID @see EPS::CommStats1
			MUuidMesh	uuid_mesh;	///< individual mesh connections, stored in a map by UUID @see EPS::CommStats1::uuid
	};
}
